@@ -1,6 +1,5 @@
 """Rasterio's GDAL/AWS environment"""
 
-from contextlib import ExitStack
 from functools import wraps, total_ordering
 from inspect import getfullargspec as getargspec
 import logging
@@ -204,6 +203,7 @@ class Env:
 
         elif 'AWS_ACCESS_KEY_ID' in os.environ and 'AWS_SECRET_ACCESS_KEY' in os.environ:
             self.session = Session.from_environ()
+            self._session_from_environ = True
 
         else:
             self.session = DummySession()
@@ -235,6 +235,9 @@ class Env:
         options.update(**kwargs)
         return Env(*args, **options)
 
+    def aws_creds_from_context_options(self):
+        return {k: v for k, v in self.context_options.items() if k.startswith('AWS_')}
+
     def credentialize(self):
         """Get credentials and configure GDAL
 
@@ -250,6 +253,15 @@ class Env:
         self.options.update(**cred_opts)
         setenv(**cred_opts)
 
+        if getattr(self, '_session_from_environ', False):
+            # if self.context_options has "AWS_*" credentials from parent context then it should
+            # always override what comes back from self.session.get_credential_options()
+            # b/c __init__ might have created a session from globally exported "AWS_*" os environ variables
+            parent_context_creds = self.aws_creds_from_context_options()
+            if not parent_context_creds: return
+            self.options.update(**parent_context_creds)
+            setenv(**parent_context_creds)
+
     def drivers(self):
         """Return a mapping of registered drivers."""
         return local._env.drivers()
@@ -471,10 +483,10 @@ class GDALVersion:
         return (self.major, self.minor) < tuple(other.major, other.minor)
 
     def __repr__(self):
-        return "GDALVersion(major={0}, minor={1})".format(self.major, self.minor)
+        return f"GDALVersion(major={self.major}, minor={self.minor})"
 
     def __str__(self):
-        return "{0}.{1}".format(self.major, self.minor)
+        return f"{self.major}.{self.minor}"
 
     @classmethod
     def parse(cls, input):
@@ -585,8 +597,8 @@ def require_gdal_version(version, param=None, values=None, is_max_version=False,
 
     version = GDALVersion.parse(version)
     runtime = GDALVersion.runtime()
-    inequality = '>=' if runtime < version else '<='
-    reason = '\n{0}'.format(reason) if reason else reason
+    inequality = ">=" if runtime < version else "<="
+    reason = f"\n{reason}" if reason else reason
 
     def decorator(f):
         @wraps(f)
@@ -596,7 +608,7 @@ def require_gdal_version(version, param=None, values=None, is_max_version=False,
 
                 if param is None:
                     raise GDALVersionError(
-                        "GDAL version must be {0} {1}{2}".format(
+                        "GDAL version must be {} {}{}".format(
                             inequality, str(version), reason))
 
                 # normalize args and kwds to dict
@@ -617,15 +629,19 @@ def require_gdal_version(version, param=None, values=None, is_max_version=False,
                         if param not in defaults or (
                                 full_kwds[param] != defaults[param]):
                             raise GDALVersionError(
-                                'usage of parameter "{0}" requires '
-                                'GDAL {1} {2}{3}'.format(param, inequality,
-                                                         version, reason))
+                                'usage of parameter "{}" requires '
+                                "GDAL {} {}{}".format(
+                                    param, inequality, version, reason
+                                )
+                            )
 
                     elif full_kwds[param] in values:
                         raise GDALVersionError(
-                            'parameter "{0}={1}" requires '
-                            'GDAL {2} {3}{4}'.format(
-                                param, full_kwds[param], inequality, version, reason))
+                            'parameter "{}={}" requires '
+                            "GDAL {} {}{}".format(
+                                param, full_kwds[param], inequality, version, reason
+                            )
+                        )
 
             return f(*args, **kwds)
 
