@@ -6,7 +6,7 @@ from cligj import (
     precision_opt, indent_opt, compact_opt, projection_geographic_opt,
     projection_mercator_opt, projection_projected_opt,
     use_rs_opt, geojson_type_feature_opt, geojson_type_bbox_opt,
-    geojson_type_collection_opt)
+)
 
 from .helpers import write_features, to_lower
 from rasterio.rio import options
