@@ -362,14 +362,14 @@ def warp(
             if "blockxsize" in out_kwargs and dst_width < int(out_kwargs["blockxsize"]):
                 del out_kwargs["blockxsize"]
                 logger.warning(
-                    "Blockxsize removed from creation options to accomodate small output width"
+                    "Blockxsize removed from creation options to accommodate small output width"
                 )
             if "blockysize" in out_kwargs and dst_height < int(
                 out_kwargs["blockysize"]
             ):
                 del out_kwargs["blockysize"]
                 logger.warning(
-                    "Blockxsize removed from creation options to accomodate small output height"
+                    "Blockxsize removed from creation options to accommodate small output height"
                 )
 
             out_kwargs.update(**creation_options)
