@@ -1,6 +1,5 @@
 """Tests of _env util module"""
 
-from numpy import isin
 import pytest
 
 from rasterio._env import (
@@ -35,10 +34,6 @@ def mock_fhs(tmpdir):
 @pytest.fixture
 def mock_debian(tmpdir):
     """A fake Debian multi-install system"""
-    tmpdir.ensure("share/gdal/3.1/gdalvrt.xsd")
-    tmpdir.ensure("share/gdal/3.2/gdalvrt.xsd")
-    tmpdir.ensure("share/gdal/3.3/gdalvrt.xsd")
-    tmpdir.ensure("share/gdal/3.4/gdalvrt.xsd")
     tmpdir.ensure("share/gdal/3.5/gdalvrt.xsd")
     tmpdir.ensure("share/gdal/3.6/gdalvrt.xsd")
     tmpdir.ensure(f"share/gdal/{gdal_version.major}.{gdal_version.minor}/gdalvrt.xsd")
@@ -79,7 +74,9 @@ def test_search_debian_gdal_data_failure(tmpdir):
 def test_search_debian_gdal_data(mock_debian):
     """Find GDAL data under Debian locations"""
     finder = GDALDataFinder()
-    assert finder.search_debian(str(mock_debian)) == str(mock_debian.join("share").join("gdal").join("{}".format(str(gdal_version))))
+    assert finder.search_debian(str(mock_debian)) == str(
+        mock_debian.join("share").join("gdal").join(f"{str(gdal_version)}")
+    )
 
 
 def test_search_gdal_data_wheel(mock_wheel):
@@ -95,7 +92,9 @@ def test_search_gdal_data_fhs(mock_fhs):
 def test_search_gdal_data_debian(mock_debian):
     """Find GDAL data under Debian locations"""
     finder = GDALDataFinder()
-    assert finder.search(str(mock_debian)) == str(mock_debian.join("share").join("gdal").join("{}".format(str(gdal_version))))
+    assert finder.search(str(mock_debian)) == str(
+        mock_debian.join("share").join("gdal").join(f"{str(gdal_version)}")
+    )
 
 
 def test_search_wheel_proj_data_failure(tmpdir):
