@@ -1,8 +1,6 @@
 import numpy as np
 import pytest
 
-from .conftest import gdal_version
-
 import rasterio
 from rasterio import (
     ubyte,
@@ -10,6 +8,7 @@ from rasterio import (
     uint16,
     uint32,
     uint64,
+    int8,
     int16,
     int32,
     int64,
@@ -68,7 +67,8 @@ def test_get_minimum_dtype():
     assert get_minimum_dtype([0, 1]) == uint8
     assert get_minimum_dtype([0, 1000]) == uint16
     assert get_minimum_dtype([0, 100000]) == uint32
-    assert get_minimum_dtype([-1, 0, 1]) == int16
+    assert get_minimum_dtype([-1, 0, 1]) == int8
+    assert get_minimum_dtype([-1, 0, 128]) == int16
     assert get_minimum_dtype([-1, 0, 100000]) == int32
     assert get_minimum_dtype([-1.5, 0, 1.5]) == float32
     assert get_minimum_dtype([-1.5e+100, 0, 1.5e+100]) == float64
@@ -76,25 +76,20 @@ def test_get_minimum_dtype():
     assert get_minimum_dtype(np.array([0, 1], dtype=np.uint)) == uint8
     assert get_minimum_dtype(np.array([0, 1000], dtype=np.uint)) == uint16
     assert get_minimum_dtype(np.array([0, 100000], dtype=np.uint)) == uint32
-    assert get_minimum_dtype(np.array([-1, 0, 1], dtype=int)) == int16
+    assert get_minimum_dtype(np.array([-1, 0, 1], dtype=int)) == int8
+    assert get_minimum_dtype(np.array([-1, 0, 128], dtype=int)) == int16
     assert get_minimum_dtype(np.array([-1, 0, 100000], dtype=int)) == int32
     assert get_minimum_dtype(np.array([-1.5, 0, 1.5], dtype=np.float64)) == float32
 
+    # Mixed type list where min/max are same type
+    assert get_minimum_dtype([0, 1.5, 5]) == float32
+
 
 def test_get_minimum_dtype__int64():
-    if gdal_version.at_least("3.5"):
-        assert get_minimum_dtype([-1, 0, 2147483648]) == int64
-    else:
-        with pytest.raises(ValueError, match="Values out of range for supported dtypes"):
-            get_minimum_dtype([-1, 0, 2147483648])
-
+    assert get_minimum_dtype([-1, 0, 2147483648]) == int64
 
 def test_get_minimum_dtype__uint64():
-    if gdal_version.at_least("3.5"):
-        assert get_minimum_dtype([0, 4294967296]) == uint64
-    else:
-        with pytest.raises(ValueError, match="Values out of range for supported dtypes"):
-            get_minimum_dtype([0, 4294967296])
+    assert get_minimum_dtype([0, 4294967296]) == uint64
 
 
 def test_can_cast_dtype():
@@ -152,8 +147,4 @@ def test_get_npdtype():
 
 
 def test__get_gdal_dtype__int64():
-    if gdal_version.at_least("3.5"):
-        assert _get_gdal_dtype("int64") == 13
-    else:
-        with pytest.raises(TypeError, match="Unsupported data type"):
-            _get_gdal_dtype("int64")
+    assert _get_gdal_dtype("int64") == 13
