@@ -1,7 +1,4 @@
-import pytest
-
 import rasterio
-from rasterio.rio.calc import _chunk_output
 from rasterio.rio.main import main_group
 
 
@@ -169,19 +166,6 @@ def test_positional_calculation_byindex(tmpdir, runner):
         assert src.read(1, window=window) == answer
 
 
-@pytest.mark.parametrize('width', [10, 791, 3000])
-@pytest.mark.parametrize('height', [8, 718, 4000])
-@pytest.mark.parametrize('count', [1, 3, 4])
-@pytest.mark.parametrize('itemsize', [1, 2, 8])
-@pytest.mark.parametrize('mem_limit', [1, 16, 64, 512])
-def test_chunk_output(width, height, count, itemsize, mem_limit):
-    work_windows = _chunk_output(width, height, count, itemsize, mem_limit=mem_limit)
-    num_windows_rows = max([i for ((i, j), w) in work_windows]) + 1
-    num_windows_cols = max([j for ((i, j), w) in work_windows]) + 1
-    assert sum((w.width for ij, w in work_windows)) == width * num_windows_rows
-    assert sum((w.height for ij, w in work_windows)) == height * num_windows_cols
-
-
 def test_bool(tmpdir, runner):
     """Check on issue #2401"""
     outfile = str(tmpdir.join("out.tif"))
