@@ -1,11 +1,9 @@
 """Tests for :py:class:`rasterio.warp.WarpedVRT`."""
 
-from __future__ import division
 import logging
 import shutil
 
 import affine
-import boto3
 import numpy
 import pytest
 
@@ -157,7 +155,7 @@ def test_wrap_file(path_rgb_byte_tif):
             -8789636.7, 2700460.0, -8524406.4, 2943560.2
         )
         assert vrt.name.startswith("WarpedVRT(")
-        assert vrt.name.endswith("tests/data/RGB.byte.tif)")
+        assert vrt.name.endswith("RGB.byte.tif)")
         assert vrt.indexes == (1, 2, 3)
         assert vrt.nodatavals == (0, 0, 0)
         assert vrt.dtypes == ("uint8", "uint8", "uint8")
